@@ -115,6 +115,7 @@ impl SimplifyExpr {
                 // 'foo'.length
                 KnownOp::Len => {
                     self.changed = true;
+
                     Expr::Lit(Lit::Num(Number {
                         value: value.chars().count() as f64,
                         span,
@@ -124,6 +125,7 @@ impl SimplifyExpr {
                 // 'foo'[1]
                 KnownOp::Index(idx) if (idx as usize) < value.len() => {
                     self.changed = true;
+
                     return if idx < 0 {
                         *undefined(span)
                     } else {
@@ -160,6 +162,7 @@ impl SimplifyExpr {
                 }
 
                 self.changed = true;
+
                 Expr::Lit(Lit::Num(Number {
                     value: elems.len() as _,
                     span,
@@ -187,6 +190,7 @@ impl SimplifyExpr {
                 }
 
                 self.changed = true;
+
                 let idx = match op {
                     KnownOp::Index(i) => i,
                     _ => unreachable!(),
@@ -304,6 +308,7 @@ impl SimplifyExpr {
                         Some(i) => {
                             let v = props.remove(i);
                             self.changed = true;
+
                             preserve_effects(
                                 span,
                                 match v {
@@ -358,6 +363,7 @@ impl SimplifyExpr {
                     Known(v) => {
                         // TODO: Optimize
                         self.changed = true;
+
                         return make_bool_expr(span, v, {
                             iter::once(left).chain(iter::once(right))
                         });
@@ -369,6 +375,7 @@ impl SimplifyExpr {
                 match $v {
                     Known(v) => {
                         self.changed = true;
+
                         return preserve_effects(
                             span,
                             Expr::Lit(Lit::Num(Number { value: v, span })),
@@ -389,6 +396,7 @@ impl SimplifyExpr {
                         let mut l = l.into_owned();
                         l.push_str(&r);
                         self.changed = true;
+
                         return Expr::Lit(Lit::Str(Str {
                             value: l.into(),
                             span,
@@ -419,6 +427,7 @@ impl SimplifyExpr {
                                 if let (Known(l), Known(r)) = (left.as_string(), right.as_string())
                                 {
                                     self.changed = true;
+
                                     return Expr::Lit(Lit::Str(Str {
                                         value: format!("{}{}", l, r).into(),
                                         span,
@@ -447,6 +456,7 @@ impl SimplifyExpr {
                             }) => match self.perform_arithmetic_op(op!(bin, "+"), &left, &right) {
                                 Known(v) => {
                                     self.changed = true;
+
                                     return preserve_effects(
                                         span,
                                         Expr::Lit(Lit::Num(Number { value: v, span })),
@@ -478,11 +488,13 @@ impl SimplifyExpr {
                             right
                         } else {
                             self.changed = true;
+
                             // 0 && $right
                             return *left;
                         }
                     } else if val {
                         self.changed = true;
+
                         // 1 || $right
                         return *left;
                     } else {
@@ -492,9 +504,11 @@ impl SimplifyExpr {
 
                     return if !left.may_have_side_effects() {
                         self.changed = true;
+
                         *node
                     } else {
                         self.changed = true;
+
                         let mut seq = SeqExpr {
                             span,
                             exprs: vec![left, node],
@@ -560,11 +574,13 @@ impl SimplifyExpr {
                 // Non-object types are never instances.
                 if is_non_obj(&left) {
                     self.changed = true;
+
                     return make_bool_expr(span, false, iter::once(right));
                 }
 
                 if is_obj(&left) && right.is_ident_ref_to(js_word!("Object")) {
                     self.changed = true;
+
                     return make_bool_expr(span, true, iter::once(left));
                 }
 
@@ -651,6 +667,7 @@ impl SimplifyExpr {
                     if left_op == op {
                         if let Known(value) = self.perform_arithmetic_op(op, &left_rhs, &right) {
                             self.changed = true;
+
                             return Expr::Bin(BinExpr {
                                 span,
                                 left: left_lhs,
@@ -764,6 +781,7 @@ impl SimplifyExpr {
                 match arg.as_bool() {
                     (_, Known(val)) => {
                         self.changed = true;
+
                         return make_bool_expr(span, !val, iter::once(arg));
                     }
                     _ => return Expr::Unary(UnaryExpr { op, arg, span }),
@@ -772,6 +790,7 @@ impl SimplifyExpr {
             op!(unary, "+") => match arg.as_number() {
                 Known(v) => {
                     self.changed = true;
+
                     return preserve_effects(
                         span,
                         Expr::Lit(Lit::Num(Number { value: v, span })),
@@ -792,10 +811,12 @@ impl SimplifyExpr {
                     ..
                 }) => {
                     self.changed = true;
+
                     return *arg;
                 }
                 Expr::Lit(Lit::Num(Number { value: f, .. })) => {
                     self.changed = true;
+
                     return Expr::Lit(Lit::Num(Number { value: -f, span }));
                 }
                 _ => {
@@ -827,6 +848,7 @@ impl SimplifyExpr {
                 if let Known(value) = arg.as_number() {
                     if value.fract() == 0.0 {
                         self.changed = true;
+
                         return Expr::Lit(Lit::Num(Number {
                             span,
                             value: if value < 0.0 {
@@ -852,7 +874,7 @@ impl SimplifyExpr {
     }
 
     /// Try to fold arithmetic binary operators
-    fn perform_arithmetic_op(&mut self, op: BinaryOp, left: &Expr, right: &Expr) -> Value<f64> {
+    fn perform_arithmetic_op(&self, op: BinaryOp, left: &Expr, right: &Expr) -> Value<f64> {
         /// Replace only if it becomes shorter
         macro_rules! try_replace {
             ($value:expr) => {{
@@ -926,13 +948,9 @@ impl SimplifyExpr {
                 // advantage of that without some kind of non-NaN proof.  So the special cases
                 // here only deal with 1*x
                 if Known(1.0) == lv {
-                    self.changed = true;
-                    // TODO: cloneTree()
                     return rv;
                 }
                 if Known(1.0) == rv {
-                    self.changed = true;
-                    // TODO: cloneTree()
                     return lv;
                 }
 
@@ -1217,10 +1235,13 @@ impl VisitMut for SimplifyExpr {
             // Remove parenthesis. This may break ast, but it will be fixed up later.
             Expr::Paren(ParenExpr { expr: e, .. }) => {
                 self.changed = true;
+
                 *expr = *e.take();
                 return;
             }
 
+            Expr::Seq(seq) if seq.exprs.is_empty() => return,
+
             Expr::Unary(..)
             | Expr::Bin(..)
             | Expr::Member(..)
@@ -1289,6 +1310,7 @@ impl VisitMut for SimplifyExpr {
                                 expr,
                             }) if expr.is_array() => {
                                 self.changed = true;
+
                                 e.extend(expr.array().unwrap().elems)
                             }
 
@@ -1336,6 +1358,7 @@ impl VisitMut for SimplifyExpr {
                         let e = &*e.args.into_iter().next().unwrap().pop().unwrap().expr;
                         if let Known(value) = e.as_string() {
                             self.changed = true;
+
                             return Expr::Lit(Lit::Str(Str {
                                 span: e.span(),
                                 value: value.into(),
@@ -1418,6 +1441,10 @@ impl VisitMut for SimplifyExpr {
 
     /// Drops unused values
     fn visit_mut_seq_expr(&mut self, e: &mut SeqExpr) {
+        if e.exprs.is_empty() {
+            return;
+        }
+
         e.visit_mut_children_with(self);
 
         let len = e.exprs.len();
@@ -1439,8 +1466,9 @@ impl VisitMut for SimplifyExpr {
                 }
 
                 Expr::Lit(..) | Expr::Ident(..) if self.in_callee => {
-                    self.changed = true;
                     if exprs.is_empty() {
+                        self.changed = true;
+
                         exprs.push(Box::new(Expr::Lit(Lit::Num(Number {
                             span: DUMMY_SP,
                             value: 0.0,
